<?php

namespace NoccyLabs\React\Http\Middleware;

use Psr\Http\Message\ResponseInterface;
use Psr\Http\Message\ServerRequestInterface;
use Psr\Log\LoggerInterface;
use React\Promise\PromiseInterface;

class LoggingMiddleware
{
    public function __construct(private LoggerInterface $logger)
    {
        
    }

    public function __invoke(ServerRequestInterface $request, ?callable $next = null): ResponseInterface|PromiseInterface
    {
        $response = $next($request);
        if ($response instanceof PromiseInterface) {
            $response->then(
                function ($r) use ($request) {
                    $this->logRequest($request, $r);
                    return $r;
                }
            );
        } else {
            $this->logRequest($request, $response);
            $response = $response->withoutHeader('x-username');
        }
        return $response;
    }

    private function logRequest(ServerRequestInterface $request, ResponseInterface $response): void
    {
        $client = $request->getServerParams()['REMOTE_ADDR'];
        if ($request->hasHeader('x-forwarded-for')) {
            $client .= ' ' . $request->getHeaderLine('x-forwarded-for');
        }

        $msg = sprintf(
            "%s %s %d %s",
            $request->getMethod(),
            $request->getUri()->getPath(),
            $response->getStatusCode(),
            $client
        );
        $this->logger->info($msg, [
            'path' => $request->getUri()->getPath(),
            'method' => $request->getMethod(),
            'status' => $response->getStatusCode(),
            'request_size' => $request->getBody()->getSize(),
            'request_type' => $request->getHeaderLine('content-type'),
            'response_size' => $response->getBody()->getSize(),
            'response_type' => $response->getHeaderLine('content-type'),
            'client' => $client,
            'username' => $response->getHeaderLine('x-username'),
        ]);
    }
}
